from time import sleep

import requests


class CommunityApi:
    def __init__(self, api_key: str):
        self.api_key = api_key

    def get_comments(self, article_url: str, limit: int = 100, offset: int = 0, sort: str = 'oldest'):
        response = requests.get(
            url='https://www.nytimes.com/svc/community/V3/requestHandler',
            params={
                'cmd': 'GetCommentsAll',
                'url': article_url,
                'sort': sort,
                'limit': limit,
                'offset': offset,
            },
        )

        sleep(1)
        return response.json()

    def get_replies(self, article_url: str, comment_sequence: int, limit: int = 100, offset: int = 0):
        response = requests.get(
            url='https://www.nytimes.com/svc/community/V3/requestHandler',
            params={
                'cmd': 'GetRepliesBySequence',
                'url': article_url,
                'commentSequence': comment_sequence,
                'limit': limit,
                'offset': offset,
            },
        )

        sleep(1)
        return response.json()



from time import sleep

import requests


class NewswireApi:
    def __init__(self, api_key: str):
        self.api_key = api_key

    def newswire(self, source: str = "nyt", section: str = "world"):
        response = requests.get(
            url=f'https://api.nytimes.com/svc/news/v3/content/{source}/{section}.json',
            params={
                'api-key': self.api_key,
            },
        )

        sleep(1)
        return response.json()



import datetime as dt
import re
from typing import List, Dict

import lxml.html
import requests
from tqdm import tqdm

from nytimes_scraper.nyt_api.api import NytApi


def fetch_articles_by_month(api: NytApi, date: dt.date, show_progress: bool = True) -> List[Dict]:
    """Fetch the article metadata for the year-month of `date`"""
    print(f'Fetching articles for {date.year}-{date.month:02d}')

    articles = api.archive.archive(date.year, date.month)['response']['docs']
    for article in tqdm(articles, unit='Article', disable=not show_progress):
        try:
            article['html'] = fetch_article_html(article['web_url'])
            article['text'] = scrape_article_text(article['html'])
        except ValueError:
            pass

    return articles


def fetch_article_html(article_url: str) -> str:
    response = requests.get(article_url)
    response.raise_for_status()
    return response.text


def scrape_article_text(article_html: str) -> str:
    doc = lxml.html.fromstring(article_html)
    text_nodes = doc.cssselect('section[name="articleBody"] > .StoryBodyCompanionColumn > :first-child > *')
    text_node_contents = [re.sub(r'[\n\s]+', ' ', node.text_content()).strip() for node in text_nodes]
    return '\n'.join(text for text in text_node_contents if text != '')


def fetch_recent_articles(api: NytApi):
    return api.newswire.newswire()['results']



from time import sleep

import requests


class ArchiveApi:
    def __init__(self, api_key: str):
        self.api_key = api_key

    def archive(self, year: int, month: int):
        response = requests.get(
            url=f'https://api.nytimes.com/svc/archive/v1/{year}/{month}.json',
            params={
                'api-key': self.api_key,
            },
        )

        sleep(1)
        return response.json()



from typing import List, Dict, Optional, Tuple

from tqdm import tqdm

from nytimes_scraper.comments.util import flatten_replies
from nytimes_scraper.nyt_api.api import NytApi


def fetch_comments(api: NytApi, article_ids_and_urls: List[Tuple[str, str]], show_progess: bool = True, pagination_size: int = 100) -> List[Dict]:
    """Fetch all comments from multiple articles, given a list of article IDs and URLs
    `[(id_1, url_1), (id_2, url_2), …]`

    The IDs are not used for processing but are added to the comment objects as an attribute for
    later reference."""

    comments = []
    for article_id, article_url in tqdm(article_ids_and_urls, unit='Article', disable=not show_progess):
        comments.extend(fetch_comments_by_article(api, article_url, article_id=article_id, pagination_size=pagination_size))

    return comments


def fetch_comments_by_article(api: NytApi, article_url: str, article_id: Optional[str] = None, pagination_size: int = 100) -> List[Dict]:
    """Fetch all comments from one specific article"""

    comments = fetch_top_level_comments(api, article_url, pagination_size=pagination_size)
    fetch_replies(api, article_url, comments, pagination_size=pagination_size)

    if article_id is not None:
        for comment in flatten_replies(comments):
            comment['articleID'] = article_id

    return comments


def fetch_top_level_comments(api: NytApi, article_url: str, pagination_size: int) -> List[Dict]:
    """Fetch all top level comments by paginating through the comment list.
    Might also include some replies."""

    comments = []
    while True:
        response = api.community.get_comments(article_url, offset=len(comments), limit=pagination_size)
        if response['status'] != 'OK':
            # some multimedia articles dont allow comments and instead throw an error here
            return []

        results = response['results']
        new_comments = results['comments']
        comments.extend(new_comments)

        if len(new_comments) < pagination_size or len(comments) >= results['totalParentCommentsFound']:
            return comments


def fetch_replies(api: NytApi, article_url: str, comments: List[Dict], pagination_size: int):
    """Fetch all replies for every comment.
    Modifies the comment objects by extending the reply lists."""

    comment_reply_queue = flatten_replies(comments)
    while len(comment_reply_queue) > 0:
        comment = comment_reply_queue.pop()

        while len(comment['replies']) < comment['replyCount']:
            response = api.community.get_replies(
                article_url=article_url,
                comment_sequence=comment['commentSequence'],
                offset=len(comment['replies']),
            )
            results = response['results']

            replies = results['comments'][0]['replies']
            comment['replies'].extend(replies)

            comment_reply_queue.extend(flatten_replies(replies))

            if len(replies) < pagination_size:
                break



from nytimes_scraper.nyt_api.archive import ArchiveApi
from nytimes_scraper.nyt_api.community import CommunityApi
from nytimes_scraper.nyt_api.newswire import NewswireApi


class NytApi:
    def __init__(self, api_key: str):
        self.archive = ArchiveApi(api_key)
        self.community = CommunityApi(api_key)
        self.newswire = NewswireApi(api_key)



from pathlib import Path
from setuptools import setup, find_packages

# The directory containing this file
here = Path(__file__).parent

# The text of the README file
readme = (here / 'README.md').read_text()

# This call to setup() does all the work
setup(
    name='nytimes-scraper-fork',
    version='1.1.3.dev4',
    description='Scrape article metadata and comments from NYTimes',
    long_description=readme,
    long_description_content_type='text/markdown',
    url='https://github.com/ietz/nytimes-scraper',
    author='Tim Pietz',
    author_email='tim@pietz.me',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'cssselect',
        'fire',
        'lxml',
        'requests',
        'tqdm',
    ],
)
